/***************************************************************************
 *   Copyright (C) 2006-2008 by Robert Keevil                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; under version 2 of the License.         *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef LIBSCROBBLE_H
#define LIBSCROBBLE_H

#ifdef WIN32
#define CURL_STATICLIB
#endif

#include <curl/curl.h>

#include <string>
#include <vector>
#include <time.h>

#define CLIENT_ID "qts"
#define CLIENT_VERSION "0.9"
#define HANDSHAKE_HOST "post.audioscrobbler.com"
#define PROTOCOL_VERSION "1.2"

#define MHOD_TITLE 1
#define MHOD_ALBUM 3
#define MHOD_ARTIST 4

#define DEBUG

using std::string;

// N.B.: The following strings are UTF-8
struct scrob_entry{
    string artist;
    string album;
    string title;
    int tracknum;
    int length;
    char played;
    int when;    // times in GMT
    string mb_track_id;
};

string MD5Digest(const char*);

class Scrobble {
private:
    string error_str;
    bool have_mb;

    // handshaking stuff
    bool handshake();
    bool need_handshake;
    string sessionid;
    string nowplay_url;
    string submit_url;
    bool real_submit();

    string username;
    string password_hash;

    bool scrob_init;

    std::vector<scrob_entry> entries;

    bool do_proxy();
    string proxy_host;
    long proxy_port;
    string proxy_user;
    string proxy_pass;
    bool proxy_winauth;

public:
    Scrobble();
    ~Scrobble();

    bool init(int, int);
    bool parse_file(const string&, int);
    bool parse_db(const string&);
    void recalc_dt(int);
    void recalc_now();
    size_t get_num_tracks();
    scrob_entry get_track(size_t);
    void remove_track(size_t);
    void update_track(scrob_entry, size_t);
    bool submit(const string&, const string&);
    string get_error_str() { return error_str; };
    bool mb_present() { return have_mb; };
    void set_proxy(const string&, long, const string&, const string&, bool);
    void cleanup_tracks(void);

    void cancel_submission();
    double get_u_tot();
    double get_u_now();

    int is_dst;
    string zonename;
    int offset;
    string offset_str();
};

#endif
